<?php
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

// Include the tables we'll be using
require_once(dirname(__FILE__) . '/../tables/character.php');
require_once(dirname(__FILE__) . '/../tables/characterseason.php');
require_once(dirname(__FILE__) . '/base.php');

ximport('Hubzero_User_Profile');

/**
 * Drwho model class for a character
 */
class DrwhoModelCharacter extends DrwhoModelBase
{
	/**
	 * Table name
	 *
	 * @var  string
	 */
	protected $_tbl_name = 'DrwhoTableCharacter';

	/**
	 * Model context
	 *
	 * @var  string
	 */
	protected $_context = 'com_drwho.character.bio';

	/**
	 * Container for cached data. Minimizes
	 * the number of queries made.
	 *
	 * @var  array
	 */
	private $_cache = array(
		'season.count'     => null,
		'season.list'      => null
	);

	/**
	 * User profile object
	 *
	 * @var  object
	 */
	private $_creator = NULL;

	/**
	 * Get the creator of this entry
	 *
	 * Accepts an optional property name. If provided
	 * it will return that property value. Otherwise,
	 * it returns the entire user object
	 *
	 * @param   string  $property  Property to return
	 * @param   mixed   $default   Default value if property not found
	 * @return  mixed
	 */
	public function creator($property=null, $default=null)
	{
		if (!($this->_creator instanceof Hubzero_User_Profile))
		{
			$this->_creator = Hubzero_User_Profile::getInstance($this->get('created_by'));
			if (!$this->_creator)
			{
				$this->_creator = new Hubzero_User_Profile();
			}
		}
		if ($property)
		{
			$property = ($property == 'id') ? 'uidNumber' : $property;
			if ($property == 'picture')
			{
				return $this->_creator->getPicture();
			}
			return $this->_creator->get($property, $default);
		}
		return $this->_creator;
	}

	/**
	 * Return a formatted timestamp
	 *
	 * @param   string $as What format to return
	 * @return  string
	 */
	public function created($as='')
	{
		switch (strtolower($as))
		{
			case 'date':
				return JHTML::_('date', $this->get('created'), "d M Y");
			break;

			case 'time':
				return JHTML::_('date', $this->get('created'), "g:i a");
			break;

			case 'relative':
				return JHTML::_('date.relative', $this->get('created'));
			break;

			default:
				if ($as)
				{
					return JHTML::_('date', $this->get('created'), $as);
				}
				return $this->get('created');
			break;
		}
	}

	/**
	 * Is the character a friend of the Doctor?
	 *
	 * @return  boolean
	 */
	public function isFriend()
	{
		if ($this->get('friend'))
		{
			return true;
		}
		return false;
	}

	/**
	 * Is the character an enemt of the Doctor?
	 *
	 * @return  boolean
	 */
	public function isEnemy()
	{
		if ($this->get('enemy'))
		{
			return true;
		}
		return false;
	}

	/**
	 * Is the character an enemt of the Doctor?
	 *
	 * @return  boolean
	 */
	public function isDoctor()
	{
		if ($this->get('doctor'))
		{
			return true;
		}
		return false;
	}

	/**
	 * Generate and return various links to the entry
	 * Link will vary depending upon action desired, such as edit, delete, etc.
	 *
	 * @param   string  $type  The type of link to return
	 * @return  string
	 */
	public function link($type='')
	{
		static $base;

		if (!isset($base))
		{
			$base = 'index.php?option=com_drwho&controller=characters';
		}

		$link = $base;

		// If it doesn't exist or isn't published
		switch (strtolower($type))
		{
			case 'edit':
				$link .= '&task=edit&id=' . $this->get('id');
			break;

			case 'delete':
				$link .= '&task=delete&id=' . $this->get('id') . '&' . JUtility::getToken() . '=1';
			break;

			case 'view':
			case 'permalink':
			default:
				$link .= '&task=view&id=' . $this->get('id');
			break;
		}

		return $link;
	}

	/**
	 * Get the character's bio
	 *
	 * @param   string   $as       Format to return state in [text, number]
	 * @param   integer  $shorten  Number of characters to shorten text to
	 * @return  string
	 */
	public function bio($as='parsed', $shorten=0)
	{
		$as = strtolower($as);
		$options = array();

		switch ($as)
		{
			case 'parsed':
				$content = $this->get('bio.parsed', null);

				if ($content === null)
				{
					$config = array();

					$content = (string) $this->get('bio', '');
					$this->importPlugin('content')->trigger('onContentPrepare', array(
						$this->_context,
						&$this,
						&$config
					));

					$this->set('bio.parsed', (string) $this->get('bio', ''));
					$this->set('bio', $content);

					return $this->bio($as, $shorten);
				}

				$options['html'] = true;
			break;

			case 'clean':
				$content = strip_tags($this->bio('parsed'));
			break;

			case 'raw':
			default:
				$content = $this->get('bio');
				$content = preg_replace('/^(<!-- \{FORMAT:.*\} -->)/i', '', $content);
			break;
		}

		if ($shorten)
		{
			$content = \Hubzero\Utility\String::truncate($content, $shorten, $options);
		}
		return $content;
	}

	/**
	 * Get a list or count of seasons
	 *
	 * @param   string   $rtrn     What type of data to return
	 * @param   array    $filters  Filters to apply to the query
	 * @param   boolean  $clear    Clear cached data?
	 * @return  mixed
	 */
	public function seasons($rtrn='list', $filters=array(), $clear=false)
	{
		if (!isset($filters['state']))
		{
			$filters['state'] = 1;
		}
		if (!isset($filters['character_id']))
		{
			$filters['character_id'] = $this->get('id');
		}

		$tbl = new DrwhoTableSeason($this->_db);

		switch (strtolower($rtrn))
		{
			case 'count':
				if (!isset($this->_cache['season.count']))
				{
					$this->_cache['season.count'] = (int) $tbl->find('count', $filters);
				}
				return $this->_cache['season.count'];
			break;

			case 'list':
			case 'results':
			default:
				if (!($this->_cache['season.list'] instanceof DrwhoModelItemList))
				{
					if ($results = $tbl->find('list', $filters))
					{
						foreach ($results as $key => $result)
						{
							$results[$key] = new DrwhoModelSeason($result);
						}
					}

					$this->_cache['season.list'] = new DrwhoModelItemList($results);
				}
				return $this->_cache['season.list'];
			break;
		}
	}

	/**
	 * Get a list or count of seasons
	 *
	 * @param   string   $rtrn     What type of data to return
	 * @param   array    $filters  Filters to apply to the query
	 * @param   boolean  $clear    Clear cached data?
	 * @return  mixed
	 */
	public function assign($seasons=array())
	{
		$remove = array();
		$exists = array();

		// Get all existing seasons and see if they're in
		// the current list. If not, mark them for removal.
		foreach ($this->seasons() as $s)
		{
			if (!in_array($s->get('id'), $seasons))
			{
				$remove[] = $s->get('id');
			}
			else
			{
				$exists[] = $s->get('id');
			}
		}

		// Loop through each season and see if the record already exists
		foreach ($seasons as $season)
		{
			$season = intval($season);
			if (!$season)
			{
				continue;
			}

			$tbl = new DrwhoTableCharacterSeason($this->_db);

			if (in_array($season, $remove))
			{
				$tbl->delete($tbl->id);
			}
			else
			{
				if (in_array($season, $exists))
				{
					continue;
				}
				$tbl->character_id = $this->get('id');
				$tbl->season_id    = $season;
				$tbl->store();
			}
		}

		return true;
	}

	/**
	 * Delete a record
	 *
	 * @return  boolean  True on success, false on error
	 */
	public function delete()
	{
		// Can't delete what doesn't exist
		if (!$this->exists())
		{
			return true;
		}

		$tbl = new DrwhoTableCharacterSeason($this->_db);
		if (!$tbl->deleteByCharacter($this->get('id')))
		{
			$this->setError($tbl->getError());
			return false;
		}

		return parent::delete();
	}
}

