<?php
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

// Include the tables we'll be using
require_once(dirname(__FILE__) . '/../tables/season.php');

// Include the models we'll be using
require_once(dirname(__FILE__) . '/doctor.php');

ximport('Hubzero_User_Profile');

/**
 * Drwho model class for a season
 */
class DrwhoModelSeason extends DrwhoModelBase
{
	/**
	 * Table name
	 *
	 * @var  string
	 */
	protected $_tbl_name = 'DrwhoTableSeason';

	/**
	 * User profile object
	 *
	 * @var  object
	 */
	private $_creator = NULL;

	/**
	 * Return a formatted timestamp for when
	 * the season started (premiere).
	 *
	 * @param   string  $as  What format to return
	 * @return  string
	 */
	public function started($as='')
	{
		return $this->_datetime('premiere_date', $as);
	}

	/**
	 * Return a formatted timestamp for when
	 * the season ended (finale).
	 *
	 * @param   string  $as  What format to return
	 * @return  string
	 */
	public function ended($as='')
	{
		return $this->_datetime('finale_date', $as);
	}

	/**
	 * Return a formatted timestamp for the 
	 * created date
	 *
	 * @param   string  $as  What format to return
	 * @return  string
	 */
	public function created($as='')
	{
		return $this->_datetime('created', $as);
	}

	/**
	 * Return a formatted timestamp
	 *
	 * @param   string  $field  Datetime field to use [premiere_date, finale_date, created]
	 * @param   string  $as     What format to return
	 * @return  string
	 */
	protected function _datetime($field, $as='')
	{
		switch (strtolower($as))
		{
			case 'date':
				return JHTML::_('date', $this->get($field), "d M Y");
			break;

			case 'time':
				return JHTML::_('date', $this->get($field), "g:i a");
			break;

			case 'relative':
				return JHTML::_('date.relative', $this->get($field));
			break;

			default:
				if ($as)
				{
					return JHTML::_('date', $this->get($field), $as);
				}
				return $this->get($field);
			break;
		}
	}

	/**
	 * Get the creator of this entry
	 *
	 * Accepts an optional property name. If provided
	 * it will return that property value. Otherwise,
	 * it returns the entire JUser object
	 *
	 * @param   string  $property  Param to return
	 * @param   mixed   $default   Value to return if property not found
	 * @return  mixed
	 */
	public function creator($property=null, $default=null)
	{
		if (!($this->_creator instanceof Hubzero_User_Profile))
		{
			$this->_creator = Hubzero_User_Profile::getInstance($this->get('created_by'));
			if (!$this->_creator)
			{
				$this->_creator = new Hubzero_User_Profile();
			}
		}
		if ($property)
		{
			$property = ($property == 'id') ? 'uidNumber' : $property;
			if ($property == 'picture')
			{
				return $this->_creator->getPicture();
			}
			return $this->_creator->get($property, $default);
		}
		return $this->_creator;
	}

	/**
	 * Get the doctor for this season
	 *
	 * @return  object
	 */
	public function doctor()
	{
		if (!isset($this->_doctor))
		{
			$this->_doctor = new DrwhoModelDoctor($this->get('doctor_id'));
		}
		return $this->_doctor;
	}

	/**
	 * Generate and return various links to the entry
	 * Link will vary depending upon action desired, such as edit, delete, etc.
	 *
	 * @param      string $type   The type of link to return
	 * @param      mixed  $params String or array of extra params to append
	 * @return     string
	 */
	public function link($type='')
	{
		static $base;

		if (!isset($base))
		{
			$base = 'index.php?option=com_drwho';
		}

		$link = $base;

		// If it doesn't exist or isn't published
		switch (strtolower($type))
		{
			case 'edit':
				$link .= '&controller=seasons&task=edit&id=' . $this->get('id');
			break;

			case 'delete':
				$link .= '&controller=seasons&task=delete&id=' . $this->get('id') . '&' . JUtility::getToken() . '=1';
			break;

			case 'view':
			case 'permalink':
			default:
				$link .= '&controller=characters&season=' . $this->get('alias');
			break;
		}

		return $link;
	}

	/**
	 * Delete a record
	 *
	 * @return  boolean  True on success, false on error
	 */
	public function delete()
	{
		// Can't delete what doesn't exist
		if (!$this->exists())
		{
			return true;
		}

		require_once(dirname(__FILE__) . '/../tables/characterseason.php');

		$tbl = new DrwhoTableCharacterSeason($this->_db);
		if (!$tbl->deleteBySeason($this->get('id')))
		{
			$this->setError($tbl->getError());
			return false;
		}

		return parent::delete();
	}
}

