<?php
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

/**
 * Dr Who database class for seasons
 */
class DrwhoTableSeason extends JTable
{
	/**
	 * Constructor
	 * 
	 * @param   object  &$db  JDatabase
	 * @return  void
	 */
	public function __construct(&$db)
	{
		parent::__construct('#__drwho_seasons', 'id', $db);
	}

	/**
	 * Validate data
	 * 
	 * @return  boolean  True if data is valid
	 */
	public function check()
	{
		$this->title = trim($this->title);
		if ($this->title == '')
		{
			$this->setError(JText::_('Please provide a title.'));
			return false;
		}

		if (!$this->alias)
		{
			$this->alias = $this->title;
		}
		$this->alias = str_replace(' ', '-', $this->alias);
		$this->alias = preg_replace("/[^a-zA-Z0-9\-]/", '', strtolower($this->alias));

		if (!$this->created_by) 
		{
			$this->created_by = JFactory::getUser()->get('id');
		}

		if (!$this->id)
		{
			$this->created = JFactory::getDate()->toSql();
		}

		return true;
	}

	/**
	 * Method to load a row from the database by primary key and bind the fields
	 * to the JTable instance properties.
	 *
	 * @param   mixed    $keys   An optional primary key value to load the row by, or an array of fields to match.  If not
	 * set the instance property value is used.
	 * @param   boolean  $reset  True to reset the default values before loading the new row.
	 * @return  boolean  True if successful. False if row not found or on error (internal error state set in that case).
	 */
	public function load($keys = null, $reset = true)
	{
		$fields = $keys;

		if (!is_numeric($keys) && is_string($keys))
		{
			$fields = array(
				'alias' => (string) $keys
			);
		}

		return parent::load($fields);
	}

	/**
	 * Get records
	 * 
	 * @param   array  $filters  Filters to build query from
	 * @return  array
	 */
	public function find($what='', $filters=array())
	{
		$what = strtolower(trim($what));

		switch ($what)
		{
			case 'count':
				$query = "SELECT COUNT(*) " . $this->_buildQuery($filters);
				$this->_db->setQuery($query);
				return $this->_db->loadResult();
			break;

			case 'one':
				$filters['start'] = 0;
				$filters['limit'] = 1;
				$result = $this->find('list', $filters);
				return $result[0];
			break;

			case 'all':
				$filters['start'] = 0;
				$filters['limit'] = 0;
				return $this->find('list', $filters);
			break;

			case 'list':
			case 'records':
			default:
				$query = "SELECT c.* " . $this->_buildQuery($filters);

				if (isset($filters['sort']) && $filters['sort'] != '') 
				{
					if (!in_array(strtoupper($filters['sort_Dir']), array('ASC', 'DESC')) || !isset($filters['sort_Dir']))
					{
						$filters['sort_Dir'] = 'ASC';
					}
					$query .= " ORDER BY " . $filters['sort'];
				} 

				if (isset($filters['limit']) && $filters['limit'] != 0) 
				{
					$query .= " LIMIT " . $filters['start'] . "," . $filters['limit'];
				}

				$this->_db->setQuery($query);
				return $this->_db->loadObjectList();
			break;
		}
	}

	/**
	 * Build a query from filters passed
	 * 
	 * @param   array   $filters  Filters to build query from
	 * @return  string  SQL
	 */
	private function _buildQuery($filters)
	{
		$query  = "FROM $this->_tbl AS c";

		$where = array();

		if (isset($filters['alias']) && $filters['alias'] != '')
		{
			$where[] = "c.alias=" . $this->_db->quote($filters['alias']);
		}
		if (isset($filters['state']) && $filters['state'] >= 0)
		{ 
			$where[] = "c.state=" . $this->_db->quote(intval($filters['state']));
		}

		if (isset($filters['created_by']) && (int) $filters['created_by'] != 0) 
		{
			$where[] = "c.created_by=" . $this->_db->quote(intval($filters['created_by']));
		}
		if (isset($filters['doctor_id']) && (int) $filters['doctor_id'] >= 0) 
		{
			$where[] = "c.doctor_id=" . $this->_db->quote(intval($filters['doctor_id']));
		}

		if (isset($filters['character_id']) && (int) $filters['character_id'] >= 0) 
		{
			$query .= " JOIN `#__drwho_character_seasons` AS s ON s.season_id=c.id";
			$where[] = "s.character_id=" . $this->_db->quote(intval($filters['character_id']));
		}

		if (isset($filters['search']) && $filters['search'] != '') 
		{
			$filters['search'] = $this->_db->quote('%' . strtolower(stripslashes($filters['search'])) . '%');
			$where[] = "LOWER(c.title) LIKE " . $filters['search'];
		}

		if (count($where) > 0)
		{
			$query .= " WHERE " . implode(" AND ", $where);
		}

		return $query;
	}
}

