<?php
// The "Show" model pulls in all other models and JTable classes
// used throughout the component.
//
// NOTE: We're using the __DIR__ constant. This is a constant
// automatically defined in PHP 5.3+. Its value is the absolute
// path up to the directory that this file is in. Using this instead
// of a Joomla-defined constant such as JPATH_COMPONENT keeps our 
// code a little more flexible, allowing us to move files or even
// entire components with fewer changes.
include_once(__DIR__ . '/models/show.php');

ximport('Hubzero_Controller');
ximport('Hubzero_Document');

// Determine which controller we're using.
//
// If no controller is pecified, we'll fall back to the "show" 
// controller, specified below as the second argument to getCmd();
$controller = JRequest::getCmd('controller', 'show');

// Make extra sure that controller exists
//
// This is an extra-paranoid check to ensure only an existing
// controller is called. If the specified controller does NOT exist
// we forcefully set the controller name to our default ("show").
//
// Another option might be to simply throw a 404 error. The code below
// has the advantage of making the experience a little smoother to the
// end user but disadvantage in that a technically-incorrect URL will
// resolve to a 202 (success) status.
//
// So, consider the behavior carefully. We're using the default option
// as it'll make development a little smoother for the moment.
if (!file_exists(__DIR__ . '/controllers/' . $controller . '.php'))
{
	$controller = 'show';
}
include_once(__DIR__ . '/controllers/' . $controller . '.php');

// Build the class name
//
// Class names are camel-cased and follow the same pattern:
//
// {Component name}{Directory name}{File name}
//
// So, for a controller with the name of "show" in this component:
//
// /com_drwho
//    /controllers
//        /show.php
//
// ... we get the final class name of "DrwhoControllerShow".
//
// Typically, directories are plural (controllers, models, tables, helpers)
// but the title used in the class name is singular.
$controllerName = 'DrwhoController' . ucfirst(strtolower($controller));

// Instantiate the controller
$component = new $controllerName();

// This detects the incoming task and executes it if it can. If no task 
// is set, it will execute a default task of "display" which maps to a 
// method of "displayTask" in the controller.
$component->execute();

// If any redirection URL is set in the controller
// this method will detect that value and redirect the CMS to it.
$component->redirect();