<?php
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

// Include the models we'll be using
require_once(__DIR__ . '/doctor.php');
require_once(__DIR__ . '/season.php');
require_once(__DIR__ . '/tardis.php');

/**
 * Drwho model class for the show
 */
class DrwhoModelShow extends \Hubzero\Object
{
	/**
	 * Container for cached data. Minimizes
	 * the number of queries made.
	 *
	 * @var  array
	 */
	private $_cache = array(
		'characters.count' => null,
		'characters.list'  => null,
		'season.count'     => null,
		'season.list'      => null
	);

	/**
	 * JRegistry
	 *
	 * @var  array
	 */
	private $_config = null;

	/**
	 * DrwhoModelTardis
	 *
	 * @var  object
	 */
	private $_tardis = null;

	/**
	 * Constructor
	 *
	 * @return  void
	 */
	public function __construct($db=null)
	{
		if (!($db instanceof JDatabase))
		{
			$db = \JFactory::getDBO();
		}
		$this->_db = $db;
	}

	/**
	 * Returns a reference to this model
	 *
	 * @return  object  DrwhoModelShow
	 */
	static function &getInstance()
	{
		static $instance;

		if (!isset($instance))
		{
			$instance = new static();
		}

		return $instance;
	}

	/**
	 * Get a list or count of characters
	 *
	 * @param   string   $rtrn     What type of data to return
	 * @param   array    $filters  Filters to apply to the query
	 * @param   boolean  $clear    Clear cached data?
	 * @return  mixed
	 */
	public function characters($rtrn='list', $filters=array(), $clear=false)
	{
		if (!isset($filters['state']))
		{
			$filters['state'] = 1;
		}

		$tbl = new DrwhoTableCharacter($this->_db);

		switch (strtolower($rtrn))
		{
			case 'count':
				if (!isset($this->_cache['characters.count']))
				{
					$this->_cache['characters.count'] = (int) $tbl->find('count', $filters);
				}
				return $this->_cache['characters.count'];
			break;

			case 'list':
			case 'results':
			default:
				if (!($this->_cache['characters.list'] instanceof \Hubzero\ItemList))
				{
					if ($results = $tbl->find('list', $filters))
					{
						foreach ($results as $key => $result)
						{
							$model = ($result->name == 'TARDIS') ? 'DrwhoModelTardis' : 'DrwhoModelCharacter';

							if ($result->doctor)
							{
								$model = 'DrwhoModelDoctor';
							}

							$results[$key] = new $model($result);
						}
					}
					$this->_cache['characters.list'] = new \Hubzero\ItemList($results);
				}
				return $this->_cache['characters.list'];
			break;
		}
	}

	/**
	 * Get a list or count of seasons
	 *
	 * @param   string   $rtrn     What type of data to return
	 * @param   array    $filters  Filters to apply to the query
	 * @param   boolean  $clear    Clear cached data?
	 * @return  mixed
	 */
	public function seasons($rtrn='list', $filters=array(), $clear=false)
	{
		if (!isset($filters['state']))
		{
			$filters['state'] = 1;
		}

		$tbl = new DrwhoTableSeason($this->_db);

		switch (strtolower($rtrn))
		{
			case 'count':
				if (!isset($this->_cache['season.count']))
				{
					$this->_cache['season.count'] = (int) $tbl->find('count', $filters);
				}
				return $this->_cache['season.count'];
			break;

			case 'list':
			case 'results':
			default:
				if (!($this->_cache['season.list'] instanceof \Hubzero\ItemList))
				{
					if ($results = $tbl->find('list', $filters))
					{
						foreach ($results as $key => $result)
						{
							$results[$key] = new DrwhoModelSeason($result);
						}
					}

					$this->_cache['season.list'] = new \Hubzero\ItemList($results);
				}
				return $this->_cache['season.list'];
			break;
		}
	}

	/**
	 * Get a TARDIS model
	 *
	 * @return  object  DrwhoModelTardis
	 */
	public function tardis()
	{
		if (!isset($this->_tardis))
		{
			$tbl = new DrwhoTableSeason($this->_db);
			if (!($result = $tbl->find('one', array('name' => 'TARDIS'))))
			{
				$result = null;
			}

			$this->_tardis = new DrwhoModelTardis($result);
		}
		return $this->_tardis;
	}

	/**
	 * Get a parameter from the component config
	 *
	 * @param   string  $property  Param to return
	 * @param   mixed   $default   Value to return if property not found
	 * @return  mixed
	 */
	public function config($property=null, $default=null)
	{
		if (!isset($this->_config))
		{
			$this->_config = JComponentHelper::getParams('com_drwho');
		}
		if ($property)
		{
			return $this->_config->get($property, $default);
		}
		return $this->_config;
	}

	/**
	 * Check a user's authorization
	 *
	 * @param   string   $action  Action to check
	 * @return  boolean  True if authorized, false if not
	 */
	public function access($action='view')
	{
		if (!$this->config()->get('access-check-done', false))
		{
			$juser = JFactory::getUser();
			if ($juser->get('guest'))
			{
				$this->config()->set('access-check-done', true);
			}
			else
			{
				$this->config()->set('access-admin-entry', $juser->authorise('core.admin', $this->get('id')));
				$this->config()->set('access-manage-entry', $juser->authorise('core.manage', $this->get('id')));
				$this->config()->set('access-delete-entry', $juser->authorise('core.manage', $this->get('id')));
				$this->config()->set('access-edit-entry', $juser->authorise('core.manage', $this->get('id')));
				$this->config()->set('access-edit-state-entry', $juser->authorise('core.manage', $this->get('id')));
				$this->config()->set('access-edit-own-entry', $juser->authorise('core.manage', $this->get('id')));

				$this->config()->set('access-check-done', true);
			}
		}
		return $this->config()->get('access-' . strtolower($action) . '-entry');
	}
}
