<?php
/**
 * Build SEF route
 * 
 * Incoming data is an associative array of querystring
 * values to build a SEF URL from.
 * Example:
 *    controller=characters&task=view&id=123
 * Gets truned into:
 *    $segments = array(
 *       'controller' => 'characters',
 *       'task'=> 'view',
 *       'id' => 123
 *    );
 * To generate:
 *    url: /characters/view/123
 * 
 * @param   array  &$query
 * @return  array 
 */
function DrwhoBuildRoute(&$query)
{
	$segments = array();

	if (!empty($query['controller'])) 
	{
		$segments[] = $query['controller'];
		unset($query['controller']);
	}
	if (!empty($query['task'])) 
	{
		$segments[] = $query['task'];
		unset($query['task']);
	}
	if (!empty($query['id'])) 
	{
		$segments[] = $query['id'];
		unset($query['id']);
	}

	return $segments;
}

/**
 * Parse SEF route
 *
 * Incoming data is an array of URL segments.
 * Example:
 *    url: /characters/view/123
 *    $segments = array('characters', 'view', '123')
 * 
 * @param   array  $segments
 * @return  array
 */
function DrwhoParseRoute($segments)
{
	$vars = array();

	if (empty($segments))
	{
		return $vars;
	}

	if (isset($segments[0])) 
	{
		$vars['controller'] = $segments[0];
	}
	if (isset($segments[1])) 
	{
		$vars['task'] = $segments[1];
	}
	if (isset($segments[2])) 
	{
		$vars['id'] = $segments[2];
	}

	return $vars;
}

