<?php
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

/**
 * Dr Who database class for characters
 */
class DrwhoTableCharacter extends JTable
{
	/**
	 * Constructor
	 * 
	 * @param   object  &$db  JDatabase
	 * @return  void
	 */
	public function __construct(&$db)
	{
		parent::__construct('#__drwho_characters', 'id', $db);
	}

	/**
	 * Validate data
	 * 
	 * @return  boolean  True if data is valid
	 */
	public function check()
	{
		$this->name = trim($this->name);
		if ($this->name == '') 
		{
			$this->setError(JText::_('Please provide a name.'));
			return false;
		}

		$this->bio = trim($this->bio);
		if ($this->bio == '') 
		{
			$this->setError(JText::_('Please provide a bio.'));
			return false;
		}

		$juser = JFactory::getUser();
		if (!$this->created_by) 
		{
			$this->created_by = $juser->get('id');
		}

		if (!$this->id)
		{
			$this->created = JFactory::getDate()->toSql();
		}

		return true;
	}

	/**
	 * Get records
	 * 
	 * @param   array  $filters  Filters to build query from
	 * @return  array
	 */
	public function find($what='', $filters=array())
	{
		$what = strtolower(trim($what));

		switch ($what)
		{
			case 'count':
				$query = "SELECT COUNT(*) " . $this->_buildQuery($filters);
				$this->_db->setQuery($query);
				return $this->_db->loadResult();
			break;

			case 'one':
				$filters['start'] = 0;
				$filters['limit'] = 1;
				$result = $this->find('list', $filters);
				return $result[0];
			break;

			case 'all':
				$filters['start'] = 0;
				$filters['limit'] = 0;
				return $this->find('list', $filters);
			break;

			case 'list':
			case 'records':
			default:
				$query = "SELECT c.* " . $this->_buildQuery($filters);

				if (isset($filters['sort']) && $filters['sort'] != '') 
				{
					if (!isset($filters['sort_Dir']) || !in_array(strtoupper($filters['sort_Dir']), array('ASC', 'DESC')))
					{
						$filters['sort_Dir'] = 'ASC';
					}
					if ($filters['sort'] == 'rand')
					{
						$filters['sort'] = 'RAND()';
						$filters['sort_Dir'] = '';
					}
					$query .= " ORDER BY " . $filters['sort'] . ($filters['sort_Dir'] ? " " . $filters['sort_Dir'] : '');
				} 

				if (isset($filters['limit']) && $filters['limit'] != 0) 
				{
					$query .= " LIMIT " . $filters['start'] . "," . $filters['limit'];
				}

				$this->_db->setQuery($query);
				return $this->_db->loadObjectList();
			break;
		}
	}

	/**
	 * Build a query from filters passed
	 * 
	 * @param   array   $filters  Filters to build query from
	 * @return  string  SQL
	 */
	private function _buildQuery($filters)
	{
		$query  = "FROM $this->_tbl AS c";

		$where = array();

		if (isset($filters['name']) && $filters['name'] != '')
		{
			$where[] = "c.name=" . $this->_db->Quote($filters['name']);
		}
		if (isset($filters['state']) && $filters['state'] >= 0)
		{ 
			$where[] = "c.state=" . $this->_db->Quote(intval($filters['state']));
		}
		if (isset($filters['doctor']) && $filters['doctor'] >= 0)
		{ 
			$where[] = "c.doctor=" . $this->_db->Quote(intval($filters['doctor']));
		}

		if (isset($filters['id>']) && (int) $filters['id>'] != 0) 
		{
			$where[] = "c.id > " . $this->_db->Quote(intval($filters['id>']));
		}
		if (isset($filters['id<']) && (int) $filters['id<'] != 0) 
		{
			$where[] = "c.id < " . $this->_db->Quote(intval($filters['id<']));
		}

		if (isset($filters['created_by']) && (int) $filters['created_by'] != 0) 
		{
			$where[] = "c.created_by=" . $this->_db->Quote(intval($filters['created_by']));
		}
		if (isset($filters['enemy']) && $filters['enemy'] >= 0) 
		{
			$where[] = "c.enemy=" . $this->_db->Quote(intval($filters['enemy']));
		}
		if (isset($filters['season']) && $filters['season'] > 0) 
		{
			$query .= " JOIN `#__drwho_character_seasons` AS s ON s.character_id=c.id";
			$where[] = "s.season_id=" . $this->_db->Quote(intval($filters['season']));
		}
		if (isset($filters['friend']) && $filters['friend'] >= 0) 
		{
			$where[] = "c.friend=" . $this->_db->Quote(intval($filters['friend']));
		}
		if (isset($filters['search']) && $filters['search'] != '') 
		{
			$filters['search'] = $this->_db->quote('%' . strtolower(stripslashes($filters['search'])) . '%');
			$where[] = "(LOWER(c.name) LIKE " . $filters['search'] . " OR LOWER(c.bio) LIKE " . $filters['search'] . ")";
		}

		if (count($where) > 0)
		{
			$query .= " WHERE " . implode(" AND ", $where);
		}

		return $query;
	}
}

